/**
* @license Apache-2.0
*
* Copyright (c) 2020 The Stdlib Authors.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

'use strict';

// MODULES //

var resolve = require( 'path' ).resolve;
var tape = require( 'tape' );
var PINF = require( '@stdlib/constants/float32/pinf' );
var NINF = require( '@stdlib/constants/float32/ninf' );
var isnanf = require( './../../../../base/assert/is-nanf' );
var tryRequire = require( '@stdlib/utils/try-require' );


// VARIABLES //

var deg2radf = tryRequire( resolve( __dirname, './../lib/native.js' ) );
var opts = {
	'skip': ( deg2radf instanceof Error )
};


// FIXTURES //

var data = require( './fixtures/julia/data.json' );


// TESTS //

tape( 'main export is a function', opts, function test( t ) {
	t.ok( true, __filename );
	t.equal( typeof deg2radf, 'function', 'main export is a function' );
	t.end();
});

tape( 'if provided `+infinity`, the function returns `+infinity`', opts, function test( t ) {
	var r = deg2radf( PINF );
	t.equal( r, PINF, 'returns +infinity' );
	t.end();
});

tape( 'if provided `-infinity`, the function returns `-infinity`', opts, function test( t ) {
	var r = deg2radf( NINF );
	t.equal( r, NINF, 'returns -infinity' );
	t.end();
});

tape( 'if provided `NaN`, the function returns `NaN`', opts, function test( t ) {
	var r = deg2radf( NaN );
	t.equal( isnanf( r ), true, 'returns NaN' );
	t.end();
});

tape( 'the function converts an angle from degrees to radians', opts, function test( t ) {
	var expected;
	var x;
	var r;
	var i;

	x = data.x;
	expected = data.expected;

	for ( i = 0; i < x.length; i++ ) {
		r = deg2radf( x[i] );
		t.equal( r, expected[i], 'returns '+expected[i]+' when provided '+x[i] );
	}
	t.end();
});

tape( 'if provided a value less than `~1.4e-45*180/pi`, the function will underflow', opts, function test( t ) {
	var r = deg2radf( 1.0e-45 );
	t.equal( r, 0.0, 'returns 0' );
	t.end();
});
